package com.docus.server.common.db.type.handler;

import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.MappedTypes;

import java.sql.*;
import java.time.LocalDateTime;

//用于兼容gbase localdatetime 转换
@MappedTypes(value = LocalDateTime.class)
public class LocalDateTimeTypeHandler extends BaseTypeHandler<LocalDateTime> {

    //驱动是否原生支持
    private Boolean isJdbcSupport;

    private boolean getIsJdbcSupport(ResultSet rs, String columnName) {
        if (isJdbcSupport == null) {
            try {
                rs.getObject(columnName, LocalDateTime.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    private boolean getIsJdbcSupport(CallableStatement cs, int columnIndex) {
        if (isJdbcSupport == null) {
            try {
                cs.getObject(columnIndex, LocalDateTime.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    private boolean getIsJdbcSupport(ResultSet rs, int columnIndex) {
        if (isJdbcSupport == null) {
            try {
                rs.getObject(columnIndex, LocalDateTime.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, LocalDateTime localDateTime, JdbcType jdbcType) throws SQLException {
        if (isJdbcSupport != null && isJdbcSupport) {
            ps.setObject(i, localDateTime);
        } else {
            ps.setObject(i, Timestamp.valueOf(localDateTime));
        }
    }

    @Override
    public LocalDateTime getNullableResult(ResultSet resultSet, String columnName) throws SQLException {
        if (getIsJdbcSupport(resultSet, columnName)) {
            return resultSet.getObject(columnName, LocalDateTime.class);
        } else {
            Timestamp timestamp = resultSet.getTimestamp(columnName);
            return timestamp == null ? null : timestamp.toLocalDateTime();
        }
    }

    @Override
    public LocalDateTime getNullableResult(ResultSet resultSet, int columnIndex) throws SQLException {
        if (getIsJdbcSupport(resultSet, columnIndex)) {
            return resultSet.getObject(columnIndex, LocalDateTime.class);
        } else {
            Timestamp timestamp = resultSet.getTimestamp(columnIndex);
            return timestamp == null ? null : timestamp.toLocalDateTime();
        }
    }

    @Override
    public LocalDateTime getNullableResult(CallableStatement callableStatement, int columnIndex) throws SQLException {
        if (getIsJdbcSupport(callableStatement, columnIndex)) {
            return callableStatement.getObject(columnIndex, LocalDateTime.class);
        } else {
            Timestamp timestamp = callableStatement.getTimestamp(columnIndex);
            return timestamp == null ? null : timestamp.toLocalDateTime();
        }
    }
}
