package com.docus.server.common.db.type.handler;

import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.MappedTypes;

import java.sql.*;
import java.time.LocalDate;

//用于兼容gbase LocalDate 转换
@MappedTypes(value = LocalDate.class)
public class LocalDateTypeHandler extends BaseTypeHandler<LocalDate> {

    //驱动是否原生支持
    private Boolean isJdbcSupport;

    private boolean getIsJdbcSupport(ResultSet rs, String columnName) {
        if (isJdbcSupport == null) {
            try {
                rs.getObject(columnName, LocalDate.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    private boolean getIsJdbcSupport(CallableStatement cs, int columnIndex) {
        if (isJdbcSupport == null) {
            try {
                cs.getObject(columnIndex, LocalDate.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    private boolean getIsJdbcSupport(ResultSet rs, int columnIndex) {
        if (isJdbcSupport == null) {
            try {
                rs.getObject(columnIndex, LocalDate.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, LocalDate localDate, JdbcType jdbcType) throws SQLException {
        if (isJdbcSupport != null && isJdbcSupport) {
            ps.setObject(i, localDate);
        } else {
            ps.setObject(i, Date.valueOf(localDate));
        }
    }

    @Override
    public LocalDate getNullableResult(ResultSet resultSet, String columnName) throws SQLException {
        if (getIsJdbcSupport(resultSet, columnName)) {
            return resultSet.getObject(columnName, LocalDate.class);
        } else {
            Date date = resultSet.getDate(columnName);
            return date == null ? null : date.toLocalDate();
        }
    }

    @Override
    public LocalDate getNullableResult(ResultSet resultSet, int columnIndex) throws SQLException {
        if (getIsJdbcSupport(resultSet, columnIndex)) {
            return resultSet.getObject(columnIndex, LocalDate.class);
        } else {
            Date date = resultSet.getDate(columnIndex);
            return date == null ? null : date.toLocalDate();
        }
    }

    @Override
    public LocalDate getNullableResult(CallableStatement callableStatement, int columnIndex) throws SQLException {
        if (getIsJdbcSupport(callableStatement, columnIndex)) {
            return callableStatement.getObject(columnIndex, LocalDate.class);
        } else {
            Date date = callableStatement.getDate(columnIndex);
            return date == null ? null : date.toLocalDate();
        }
    }
}
