package com.docus.server.common.db.type.handler;

import org.apache.ibatis.type.BaseTypeHandler;
import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.MappedTypes;

import java.sql.*;
import java.time.LocalTime;

//用于兼容gbase LocalTime 转换
@MappedTypes(value = LocalTime.class)
public class LocalTimeTypeHandler extends BaseTypeHandler<LocalTime> {

    //驱动是否原生支持
    private Boolean isJdbcSupport;

    private boolean getIsJdbcSupport(ResultSet rs, String columnName) {
        if (isJdbcSupport == null) {
            try {
                rs.getObject(columnName, LocalTime.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    private boolean getIsJdbcSupport(CallableStatement cs, int columnIndex) {
        if (isJdbcSupport == null) {
            try {
                cs.getObject(columnIndex, LocalTime.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    private boolean getIsJdbcSupport(ResultSet rs, int columnIndex) {
        if (isJdbcSupport == null) {
            try {
                rs.getObject(columnIndex, LocalTime.class);
                isJdbcSupport = true;
            } catch (Exception e) {
                isJdbcSupport = false;
            }
        }
        return isJdbcSupport;
    }

    @Override
    public void setNonNullParameter(PreparedStatement ps, int i, LocalTime LocalTime, JdbcType jdbcType) throws SQLException {
        if (isJdbcSupport != null && isJdbcSupport) {
            ps.setObject(i, LocalTime);
        } else {
            ps.setObject(i, Time.valueOf(LocalTime));
        }
    }

    @Override
    public LocalTime getNullableResult(ResultSet resultSet, String columnName) throws SQLException {
        if (getIsJdbcSupport(resultSet, columnName)) {
            return resultSet.getObject(columnName, LocalTime.class);
        } else {
            Time time = resultSet.getTime(columnName);
            return time == null ? null : time.toLocalTime();
        }
    }

    @Override
    public LocalTime getNullableResult(ResultSet resultSet, int columnIndex) throws SQLException {
        if (getIsJdbcSupport(resultSet, columnIndex)) {
            return resultSet.getObject(columnIndex, LocalTime.class);
        } else {
            Time time = resultSet.getTime(columnIndex);
            return time == null ? null : time.toLocalTime();
        }
    }

    @Override
    public LocalTime getNullableResult(CallableStatement callableStatement, int columnIndex) throws SQLException {
        if (getIsJdbcSupport(callableStatement, columnIndex)) {
            return callableStatement.getObject(columnIndex, LocalTime.class);
        } else {
            Time time = callableStatement.getTime(columnIndex);
            return time == null ? null : time.toLocalTime();
        }
    }
}
